"""
License
-------
    The MIT License (MIT)

    Copyright (c) 2018 Snappy2 Project

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

Created on Sat Dec 16 22:46:28 2017

@author: Ahmad Barqawi
@Source: Github.com/Barqawiz
"""
class Constants:
    # draw based on position
    pos_eyes = 'eyes'
    pos_ears = 'ears'
    pos_head = 'head'
    pos_must = 'mustache'
    pos_noise = 'noise'
    pos_lips = 'lips'


class Overlays:
    # draw based on objects
    draw_mus = ['mus1.png', 'mus2.png', 'mus3.png']
    draw_glasses = ['sunglasses1.png', 'sunglasses2.png']
    draw_hat = ['hat1.png', 'hat2.png']
    draw_ears = ['ears_bear.png', 'ears_cat.png', 'ears_teady.png']
